@@ -17,7 +17,7 @@ use Exporter ();
 
 use vars qw { $VERSION @ISA %EXPORT_TAGS };
 BEGIN {
-    $VERSION     = '2.04_01';
+    $VERSION     = '2.04_02';
     @ISA         = 'Exporter';
     %EXPORT_TAGS = (
         ALL => [ qw{
@@ -37,25 +37,21 @@ BEGIN {
 
 Exporter::export_ok_tags('ALL');
 
-# NEED TO COVER MANY MORE CASES HERE!!!
-my $RE_ALLOW_PAT = qr#
-    ( (?:\*\*|&&|\|\||<<|>>|//|[-+*x/%^&|.])=?
-    | [!=]~
+our $RE_PREREGEX_PAT = qr#(
+    [!=]~
+    | split|grep|map
+)#x;
+our $RE_EXPR_PAT = qr#(
+    (?:\*\*|&&|\|\||<<|>>|//|[-+*x%^&|.])=?
+    | /(?:[^/])
     | =(?!>)
-    | split|grep|map|return
+    | return
     | [\(\[]
-    )
-#x;
-my $RE_NUM = qr/\s*[+\-.0-9][+\-.0-9e]*/i; # numerical constant
+)#x;
+our $RE_NUM = qr/\s*[+\-.0-9][+\-.0-9e]*/i; # numerical constant
 
-## no critic (Subroutines::ProhibitSubroutinePrototypes)
-
-# PROTOTYPES
-
-sub _match_variable($$);
-sub _match_quotelike($$$$);
-
-my %ref2patvalid; # is a quotelike /.../ pattern valid at this point for given textref?
+our %ref2slashvalid; # is quotelike /.../ pattern valid here for given textref?
+our %ref2qmarkvalid; # is quotelike ?...? pattern valid here for given textref?
 
 # HANDLE RETURN VALUES IN VARIOUS CONTEXTS
 
@@ -110,6 +106,7 @@ sub _succeed {
 }
 
 # BUILD A PATTERN MATCHING A SIMPLE DELIMITED STRING
+## no critic (Subroutines::ProhibitSubroutinePrototypes)
 
 sub gen_delimited_pat($;$)  # ($delimiters;$escapes)
 {
@@ -143,7 +140,7 @@ sub gen_delimited_pat($;$)  # ($delimiters;$escapes)
 sub extract_delimited (;$$$$)
 {
     my $textref = defined $_[0] ? \$_[0] : \$_;
-    $ref2patvalid{$textref} = 1 if !pos($$textref); # reset
+    $ref2slashvalid{$textref} = 1, $ref2qmarkvalid{$textref} = 0 if !pos($$textref); # reset
     my $wantarray = wantarray;
     my $del  = defined $_[1] ? $_[1] : qq{\'\"\`};
     my $pre  = defined $_[2] ? $_[2] : '\s*';
@@ -186,7 +183,7 @@ sub _eb_delims {
 sub extract_bracketed (;$$$)
 {
     my $textref = defined $_[0] ? \$_[0] : \$_;
-    $ref2patvalid{$textref} = 1 if !pos($$textref); # reset
+    $ref2slashvalid{$textref} = 1, $ref2qmarkvalid{$textref} = 0 if !pos($$textref); # reset
     my $ldel = defined $_[1] ? $_[1] : '{([<';
     my $pre  = defined $_[2] ? qr/\G$_[2]/ : qr/\G\s*/;
     my $wantarray = wantarray;
@@ -269,9 +266,9 @@ sub _match_bracketed    # $textref, $pre, $ldel, $qdel, $quotelike, $rdel
             pos $$textref = $startpos;
             return;
         }
-        elsif ($quotelike && _match_quotelike($textref,qr/\G()/,undef,undef))
+        elsif ($quotelike && _match_quotelike($textref,qr/\G()/,$ref2slashvalid{$textref},$ref2qmarkvalid{$textref}))
         {
-            $ref2patvalid{$textref} = 1; # back-compat
+            $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = 1; # back-compat
             next;
         }
 
@@ -310,7 +307,7 @@ my $et_default_ldel = '<\w+(?:' . gen_delimited_pat(q{'"}) . '|[^>])*>';
 sub extract_tagged (;$$$$$) # ($text, $opentag, $closetag, $pre, \%options)
 {
     my $textref = defined $_[0] ? \$_[0] : \$_;
-    $ref2patvalid{$textref} = 1 if !pos($$textref); # reset
+    $ref2slashvalid{$textref} = 1, $ref2qmarkvalid{$textref} = 0 if !pos($$textref); # reset
     my $ldel    = $_[1];
     my $rdel    = $_[2];
     my $pre     = defined $_[3] ? qr/\G$_[3]/ : qr/\G\s*/;
@@ -460,7 +457,7 @@ sub extract_variable (;$$)
 {
     my $textref = defined $_[0] ? \$_[0] : \$_;
     return ("","","") unless defined $$textref;
-    $ref2patvalid{$textref} = 1 if !pos($$textref); # reset
+    $ref2slashvalid{$textref} = 1, $ref2qmarkvalid{$textref} = 0 if !pos($$textref); # reset
     my $pre  = defined $_[1] ? qr/\G$_[1]/ : qr/\G\s*/;
 
     my @match = _match_variable($textref,$pre);
@@ -471,7 +468,7 @@ sub extract_variable (;$$)
                     @match[2..3,4..5,0..1];        # MATCH, REMAINDER, PREFIX
 }
 
-sub _match_variable($$)
+sub _match_variable
 {
 #  $#
 #  $^
@@ -519,6 +516,7 @@ sub _match_variable($$)
         next if $$textref =~ m/\G\s*->\s*\w+(?![{([])/gc;
         last;
     }
+    $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = 0;
 
     my $endpos = pos($$textref);
     return ($startpos, $varpos-$startpos,
@@ -549,7 +547,7 @@ sub _ec_delims {
 sub extract_codeblock (;$$$$$)
 {
     my $textref = defined $_[0] ? \$_[0] : \$_;
-    $ref2patvalid{$textref} = 1 if !pos($$textref); # reset
+    $ref2slashvalid{$textref} = 1, $ref2qmarkvalid{$textref} = 0 if !pos($$textref); # reset
     my $wantarray = wantarray;
     my $ldel_inner = defined $_[1] ? $_[1] : '{';
     my $pre = !defined $_[2] ? qr/\G\s*/ : qr/\G$_[2]/;
@@ -589,13 +587,13 @@ sub _match_codeblock
     my $closing = $1;
        $closing =~ tr/([<{/)]>}/;
     my $matched;
-    my $patvalidref = \$ref2patvalid{$textref};
-    $$patvalidref = 1 if !defined $$patvalidref or !$startpos; # default, or reset
+    $ref2slashvalid{$textref} = 1, $ref2qmarkvalid{$textref} = 0
+      if !pos($$textref) or !defined $ref2slashvalid{$textref}; # default, or reset
     while (pos($$textref) < length($$textref))
     {
         if ($rd && $$textref =~ m#\G(\Q(?)\E|\Q(s?)\E|\Q(s)\E)#gc)
         {
-            $$patvalidref = 0;
+            $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = 0;
             next;
         }
 
@@ -618,21 +616,21 @@ sub _match_codeblock
         }
 
         if (_match_variable($textref,qr/\G\s*/) ||
-            _match_quotelike($textref,qr/\G\s*/,undef,undef) )
+            _match_quotelike($textref,qr/\G\s*/,$ref2slashvalid{$textref},$ref2qmarkvalid{$textref}) )
         {
-            $$patvalidref = 0;
+            $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = 0;
             next;
         }
 
-        if ($$textref =~ m#\G\s*(?!$ldel_inner)$RE_ALLOW_PAT#gc)
+        if ($$textref =~ m#\G\s*(?!$ldel_inner)(?:$RE_PREREGEX_PAT|$RE_EXPR_PAT)#gc)
         {
-            $$patvalidref = 1;
+            $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = 1;
             next;
         }
 
         if ( _match_codeblock($textref, qr/\G\s*/, $ldel_inner, qr/\G\s*($rdel_inner)/, $ldel_inner, $rdel_inner, $rd) )
         {
-            $$patvalidref = 1;
+            $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = 1;
             next;
         }
 
@@ -645,7 +643,7 @@ sub _match_codeblock
             last;
         }
 
-        $$patvalidref = 0;
+        $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = 0;
         $$textref =~ m/\G\s*(\w+|[-=>]>|.|\Z)/gc;
     }
     continue { $@ = undef }
@@ -681,11 +679,11 @@ my %mods   = (
 sub extract_quotelike (;$$)
 {
     my $textref = $_[0] ? \$_[0] : \$_;
-    $ref2patvalid{$textref} = 1 if !pos($$textref); # reset
+    $ref2slashvalid{$textref} = 1, $ref2qmarkvalid{$textref} = 0 if !pos($$textref); # reset
     my $wantarray = wantarray;
     my $pre  = defined $_[1] ? qr/\G$_[1]/ : qr/\G\s*/;
 
-    my @match = _match_quotelike($textref,$pre,undef,undef);
+    my @match = _match_quotelike($textref,$pre,$ref2slashvalid{$textref},$ref2qmarkvalid{$textref});
     return _fail($wantarray, $textref) unless @match;
     return _succeed($wantarray, $textref,
                     $match[2], $match[18]-$match[2],    # MATCH
@@ -696,15 +694,14 @@ sub extract_quotelike (;$$)
     );
 };
 
-sub _match_quotelike($$$$)      # ($textref, $prepat, ignore rest)
+my %maybe_quote = map +($_=>1), qw(" ' `);
+sub _match_quotelike
 {
-    my ($textref, $pre) = @_;
-    my $patvalidref = \$ref2patvalid{$textref};
-    $$patvalidref = 1 if !defined $$patvalidref or !pos($$textref); # default, or reset
-    my $allow_slash_match = my $allow_qmark_match = $$patvalidref;
+    my ($textref, $pre, $allow_slash_match, $allow_qmark_match) = @_;
+    $ref2slashvalid{$textref} = 1, $ref2qmarkvalid{$textref} = 0
+      if !pos($$textref) or !defined $ref2slashvalid{$textref}; # default, or reset
 
     my ($textlen,$startpos,
-        $oppos,
         $preld1pos,$ld1pos,$str1pos,$rd1pos,
         $preld2pos,$ld2pos,$str2pos,$rd2pos,
         $modpos) = ( length($$textref), pos($$textref) = pos($$textref) || 0 );
@@ -717,15 +714,13 @@ sub _match_quotelike($$$$)      # ($textref, $prepat, ignore rest)
                  pos $$textref;
         return;
     }
-    $oppos = pos($$textref);
-
+    my $oppos = pos($$textref);
     my $initial = substr($$textref,$oppos,1);
-
-    if ($initial && $initial =~ m|^[\"\'\`]|
-                 || $allow_slash_match && $initial =~ m|^/|
-                 || $allow_qmark_match && $initial =~ m|^\?|)
+    if ($initial && $maybe_quote{$initial}
+                 || $allow_slash_match && $initial eq '/'
+                 || $allow_qmark_match && $initial eq '?')
     {
-        unless ($$textref =~ m/ \Q$initial\E [^\\$initial]* (\\.[^\\$initial]*)* \Q$initial\E /gcsx)
+        unless ($$textref =~ m/\G \Q$initial\E [^\\$initial]* (\\.[^\\$initial]*)* \Q$initial\E /gcsx)
         {
             _failmsg qq{Did not find closing delimiter to match '$initial' at "} .
                          substr($$textref, $oppos, 20) .
@@ -743,7 +738,7 @@ sub _match_quotelike($$$$)      # ($textref, $prepat, ignore rest)
         }
 
         my $endpos = pos($$textref);
-        $$patvalidref = 0;
+        $ref2qmarkvalid{$textref} = $ref2slashvalid{$textref} = 0;
         return (
             $startpos,  $oppos-$startpos,       # PREFIX
             $oppos,     0,                      # NO OPERATOR
@@ -799,7 +794,7 @@ sub _match_quotelike($$$$)      # ($textref, $prepat, ignore rest)
         $rd1pos = pos($$textref);
         $$textref =~ m{\Q$label\E\n}gc;
         $ld2pos = pos($$textref);
-        $$patvalidref = 0;
+        $ref2qmarkvalid{$textref} = $ref2slashvalid{$textref} = 0;
         return (
             $startpos,  $oppos-$startpos,       # PREFIX
             $oppos,     length($op),            # OPERATOR
@@ -894,7 +889,7 @@ sub _match_quotelike($$$$)      # ($textref, $prepat, ignore rest)
 
     $$textref =~ m/\G($mods{$op})/gc;
     my $endpos = pos $$textref;
-    $$patvalidref = 0;
+    $ref2qmarkvalid{$textref} = $ref2slashvalid{$textref} = 0;
 
     return (
         $startpos,      $oppos-$startpos,       # PREFIX
@@ -919,18 +914,22 @@ my %ref_not_regex = map +($_=>1), qw(CODE Text::Balanced::Extractor);
 
 sub _update_patvalid {
     my ($textref, $text) = @_;
-    if ($ref2patvalid{$textref} && $text =~ m/$RE_NUM\s*$/)
+    if ($ref2slashvalid{$textref} && $text =~ m/(?:$RE_NUM|[\)\]])\s*$/)
     {
-        $ref2patvalid{$textref} = 0;
-    } elsif (!$ref2patvalid{$textref} && $text =~ m/$RE_ALLOW_PAT\s*$/)
+        $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = 0;
+    } elsif (!$ref2slashvalid{$textref} && $text =~ m/$RE_PREREGEX_PAT\s*$/)
     {
-        $ref2patvalid{$textref} = 1;
+        $ref2slashvalid{$textref} = $ref2qmarkvalid{$textref} = 1;
+    } elsif (!$ref2slashvalid{$textref} && $text =~ m/$RE_EXPR_PAT\s*$/)
+    {
+        $ref2slashvalid{$textref} = 1;
+        $ref2qmarkvalid{$textref} = 0;
     }
 }
 sub extract_multiple (;$$$$)    # ($text, $functions_ref, $max_fields, $ignoreunknown)
 {
     my $textref = defined($_[0]) ? \$_[0] : \$_;
-    $ref2patvalid{$textref} = 1 if !pos($$textref); # reset
+    $ref2slashvalid{$textref} = 1, $ref2qmarkvalid{$textref} = 0 if !pos($$textref); # reset
     my $posbug = pos;
     my ($lastpos, $firstpos);
     my @fields = ();
